// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library do2template.nodevalidator;

import 'dart:html';

class Do2NodeValidator implements NodeValidator {

  static final Set<String> _allowedElements = new Set.from([
      'DO2TEMPLATE',
      ]);

  static const _standardAttributes = const <String>[
      '*::do2repeat',
      '*::do2repeat-field',
      '*::do2repeat-id',
      ];

  static final Map<String, Function> _attributeValidators = {};

  Do2NodeValidator(){
      if (_attributeValidators.isEmpty) {
          for (var attr in _standardAttributes) {
              _attributeValidators[attr] = _standardAttributeValidator;
          }
      }
  }

  bool allowsElement(Element element) {
      return _allowedElements.contains(element.tagName);
  }

  bool allowsAttribute(Element element, String attributeName, String value) {
      var tagName = element.tagName;
      var validator = _attributeValidators['$tagName::$attributeName'];
      if (validator == null) {
          validator = _attributeValidators['*::$attributeName'];
      }
      if (validator == null) {
          return false;
      }
      return validator(element, attributeName, value, this);
  }

  static bool _standardAttributeValidator(Element element, String attributeName, String value, Do2NodeValidator context) {
      return true;
  }
}